/* -----------------------------------------------------------------------------
   (1) Data
   -------------------------------------------------------------------------- */

// required packages

which estout
which xtserial
which xtistest
which xtcd2

// export folder
cap mkdir "outputs"

/* -----------------------------------------------------------------------------
   Data preparation
   -------------------------------------------------------------------------- */

// EP elections turnout, Feb 20, 2023
insheet using "data/2023-EUP-ep-elections.tsv", clear

// coerce VAP turnout to numeric (missing values NA -> .)
destring vap_turnout, force replace

// alternative measure of CV
gen cv2 = cv
replace cv2 = 0 if inlist(iso3c, "BGR", "ITA") | (iso3c == "GRC" & year > 2000)
li iso3c year cv* if cv != cv2 // n = 8 differences

// first-order status of last national election
replace lne_fo = 0
replace lne_fo = inlist(lne_type, "parliament", "president")

// 5 groups: parl (baseline) / pres / local+reg / ep / refe.
gen lne_type5:lne_type5 = .
replace lne_type5 = 1 if lne_type == "parliament"
replace lne_type5 = 2 if lne_type == "president"
replace lne_type5 = 3 if lne_type == "subnational"
replace lne_type5 = 4 if lne_type == "ep"
replace lne_type5 = 5 if lne_type == "referendum"
la def lne_type5 ///
	1 "Preceding election = Parliament" ///
	2 "Preceding election = Presidential" ///
	3 "Preceding election = Subnational" ///
	4 "Preceding election = EP" ///
	5 "Preceding election = Referendum", replace
tab lne_type lne_type5

// exclude 3 observations where last election = EP
clonevar lne_type4 = lne_type5 if lne_type5 != 4

/* -----------------------------------------------------------------------------
   Variable labels
   -------------------------------------------------------------------------- */

la var iso3c "Country code (ISO-3C)"
la var year "Election year"
// main DV
la var voter_turnout "EP election turnout (% registered voters)"
// alt. DV
la var vap_turnout "EP election turnout (% voting-age pop.)"
// unweighted, not used in models:
la var ne_count "Num. of preceding electoral rounds"
la var ne_count2 "Num. of preceding electoral contests"
// weighted, used in models:
la var ne_weighted "Weighted preceding electoral rounds"
la var ne_weighted2 "Weighted preceding electoral contests"
la var ne_months "Time since last vote (sq. root of months)"
la var tt_next "Time to next vote (sq. root of months)"
la var lne_fo "First-order preceding election (FOPE)" // parl. or pres.
la var cv "Compulsory voting"
la var cv2 "Compulsory voting (alt. measure)" // alt. measure
la var concurrent "Concurrent election"
la var first_ep "First EP election (0/1)"
la var closeness "Electoral competitiveness"
la var lne_type "Preceding election type"
la var lne_type5 "Preceding election type (5 groups)"
la var lne_type4 "Preceding election type (excl. EP)"
la var closeness "Electoral competitiveness"
// not used:
la var lne_turnout "FOPE turnout (%)" // parl. or pres.
la var n_lists "Number of electoral lists"
la var enp_votes "Effective number of parties" // votes, not seats

// drop unused predictors
drop lne_turnout n_lists enp_votes
// drop additional unused columns
drop lne_parl lne_fo2 concurrent_* tt_next? ne_days? ne_months? 

// final dataset
keep iso3c year *_turnout ne_* lne_* concurrent tt_next closeness cv* first_ep
d

/* -----------------------------------------------------------------------------
   Descriptive statistics -- Table 1 in paper and B1 in appendix
   -------------------------------------------------------------------------- */

gl vars "voter_turnout vap_turnout"
gl vars "$vars ne_months tt_next ne_weighted ne_weighted2 lne_fo"
// lne_type_*
gl vars "$vars concurrent cv closeness"

eststo clear

// descriptives
estpost tabstat $vars, columns(statistics) statistics(n mean sd min max)
esttab . using "outputs/tbl-01-descriptives.rtf", ///
	cells("count mean(fmt(2)) sd(fmt(2)) min(fmt(2)) max(fmt(2))") ///
	lab nomti nonum noobs replace

// correlations
estpost cor $vars, mat
esttab using "outputs/tbl-03-correlations.rtf", ///
	unstack not nostar noobs nonote b(2) lab replace

/* -----------------------------------------------------------------------------
   Panel structure and tests
   -------------------------------------------------------------------------- */

cap log close
cap log using "outputs/log-01-data.log", replace

// group variable
encode iso3c, gen(cty)
sort iso3c year

// time index (serializes the election years)
by iso3c: gen t = _n

// panel structure
xtset cty year

// 15 distinct EP election years over 41 years
xtdes

// number of observations by panel 
egen cases = count(t), by(iso3c)
tabstat cases if t == 1, s(n mean p50) // mean 6.25, median 6

// Wooldridge autocorrelation test
// note: won't run on actual panel structure with year gaps
xtset cty t
xtserial voter_turnout if cases > 2
assert r(p) < 0.05

// Inoue-Solon autocorrelation test (on unbalanced panel)
// note: fails to find serial correlation on actual panel structure with gaps
xtset cty t
xtistest voter_turnout, lags(1)
assert r(pvalue1) < 0.05
// no second-order autocorrelation
xtistest voter_turnout, lags(2)
assert r(pvalue1) > 0.05

// Fisher-type, augmented Dickey-Fuller nonstationarity test
// note: won't run when T < 3
// note: won't run on actual panel structure with year gaps
xtset cty t
xtunitroot fisher voter_turnout if cases > 3, dfuller lags(1)

// cross-sectional dependence
xtset cty year
xtcd2 voter_turnout

cap log close

// lagged DV (not used, small T)
// by iso3c: gen voter_turnout_lag = voter_turnout[ _n-1 ]

// done
