/* -----------------------------------------------------------------------------
   (4) Models
   -------------------------------------------------------------------------- */

// required packages

which estout
which coefplot
which xtcsd
which xttest3

// graph scheme
which scheme-modern.scheme

// export folder
cap mkdir "outputs"

/* -----------------------------------------------------------------------------
   Preprocessing
   -------------------------------------------------------------------------- */

run "00-dependencies.do"
run "01-data.do"
run "02-plots.do"
run "03-globals.do"

/* -----------------------------------------------------------------------------
   Presets
   -------------------------------------------------------------------------- */

gl plots 0 // toggle to 1 to show residuals-versus-fitted plots

gl model_note "Fixed-effects, with clustered standard errors in brackets."
gl model_note "$model_note Constant term and year dummies omitted."
gl model_note "$model_note The baseline for preceding election type is"
gl model_note "$model_note set to parliamentary elections."

/* -----------------------------------------------------------------------------
   Start clean and log output
   -------------------------------------------------------------------------- */

est clear
eststo clear

cap log close
cap log using "outputs/log-02-models.log", replace

/* -----------------------------------------------------------------------------
   Models 1-4, pooled OLS, clustered SEs (for consistency checks)
   -------------------------------------------------------------------------- */

eststo ols1: reg $eq1 , vce(cl cty)
eststo ols2: reg $eq2 , vce(cl cty)
eststo ols3: reg $eq3 , vce(cl cty)
eststo ols4: reg $eq4 , vce(cl cty)

/* -----------------------------------------------------------------------------
   Models 1-4, fixed effects
   -------------------------------------------------------------------------- */

// fixed-effects, unclustered
xtset cty year
eststo fe1: xtreg $eq1 , fe
eststo fe2: xtreg $eq2 , fe
eststo fe3: xtreg $eq3 , fe
eststo fe4: xtreg $eq4 , fe

forv i = 1/4 {
	qui est restore fe`i' 
	qui xttest3
	assert r(p) < 0.05
}

// with clustered SEs
eststo fc1: xtreg $eq1 , fe vce(cl cty)
eststo fc2: xtreg $eq2 , fe vce(cl cty)
eststo fc3: xtreg $eq3 , fe vce(cl cty)
eststo fc4: xtreg $eq4 , fe vce(cl cty)

// check consistency and bootstrapped estimates for stat. sig. predictors
esttab ols1 fe1 fc1, $scrn
wildbootstrap xtreg $eq1 , fe ///
	coef(ne_weighted concurrent tt_next cv)

esttab ols2 fe2 fc2, $scrn
wildbootstrap xtreg $eq2 , fe ///
	coef(2.lne_type4 concurrent tt_next closeness)

esttab ols3 fe3 fc3, $scrn
wildbootstrap xtreg $eq3 , fe ///
	coef(ne_weighted2 concurrent tt_next closeness)

esttab ols4 fe4 fc4, $scrn
wildbootstrap xtreg $eq4 , fe ///
	coef(2.lne_type4 ne_weighted2 concurrent tt_next closeness)

// linear hypothesis test for year dummies
forv i = 1/4 {
	qui est restore fc`i'
	qui testparm i.year
	assert r(p) < 0.1
}

// residuals-versus-fitted plots
forv i = 1/4 {
	qui est restore fc`i'
	cap drop yhat
	predict yhat, xb
	cap drop r
	predict r, e
	tw (sc r yhat, name(fe_rvf`i', replace) ms(i) mlab(cty))(lowess r yhat)
	gr export "outputs/fig-rvf-fe-m`i'.png", replace
}

// Pesaran test for cross-sectional independence
// (Models 1 and 3 only, as Models 2 and 4 are too unbalanced for the test)
// (note: won't run on actual panel structure with year gaps)
xtset cty t
foreach i in 1 3 {
	qui est restore fe`i'
	qui xtcsd, pes
	assert r(p) > 0.05 // accept H0
}

// export results
esttab fc? using "outputs/tbl-02A-main-models.rtf", ///
		sca(N_g r2_w rmse) sfmt(2) ///
		indicate("Year dummies=**year") ///
		d(_cons) addn("$model_note") ///
		mti("Model 1" "Model 2" "Model 3" "Model 4") ///
		$xprt

// with clustered SEs, restricted equations
eststo fr1: xtreg $eqr1, fe vce(cl cty)
eststo fr2: xtreg $eqr2, fe vce(cl cty)
eststo fr3: xtreg $eqr3, fe vce(cl cty)
eststo fr4: xtreg $eqr4, fe vce(cl cty)
eststo fr5: xtreg $eqr5 if lne_type5 != 4, fe vce(cl cty)

// export results
esttab fr? using "outputs/tbl-02B-restricted-models.rtf", ///
		sca(N_g r2_w rmse) sfmt(2) ///
		indicate("Year dummies=**year") ///
		d(_cons) addn("$model_note") ///
		mti("Model 5" "Model 6" "Model 7" "Model 8" "Model 9") ///
		$xprt

/* -----------------------------------------------------------------------------
   (1-4) Models 1-4, Prais-Winsten and other alternative specifications
   -------------------------------------------------------------------------- */

// Prais-Winsten (clustered SEs)
// -----------------------------
forv i = 1/4 {
	// remove year dummies from equation
	gl pw`i' = regexr("${eq`i'}", "i.year", "")
}
xtset cty year
eststo pw1: prais $pw1, vce(cl cty) rhotype(reg)
eststo pw2: prais $pw2, vce(cl cty) rhotype(reg)
eststo pw3: prais $pw3, vce(cl cty) rhotype(reg)
eststo pw4: prais $pw4, vce(cl cty) rhotype(reg)

if $plots {
	
	// compare with `xtpcse` estimation, and show estimates with year dummies
	// included alongside the AR(1) term
	forv i = 1/4 {
		xtset cty year
		eststo pcse`i': xtpcse ${pw`i'}, c(ar1)
		// serialize years
		xtset cty t
		eststo pw`i'yd: prais ${eq`i'}, vce(cl cty) rhotype(reg)
		eststo pcse`i'yd: xtpcse ${eq`i'}, c(ar1)
	}
	forv i = 1/4 {

		// including all specifications for reference
		coefplot ///
			(fc`i', if(@ll<0 & @ul>0)) ///
			(fc`i', if(@ll>0 | @ul<0)) || ///
			(pw`i', if(@ll<0 & @ul>0)) ///
			(pw`i', if(@ll>0 | @ul<0)) || ///
			(pcse`i', if(@ll<0 & @ul>0)) ///
			(pcse`i', if(@ll>0 | @ul<0)) || ///
			(pw`i'yd, if(@ll<0 & @ul>0)) ///
			(pw`i'yd, if(@ll>0 | @ul<0)) || ///
			(pcse`i'yd, if(@ll<0 & @ul>0)) ///
			(pcse`i'yd, if(@ll>0 | @ul<0)) || ///
			, nooffset drop(_cons *.year) ///
			xlabel(1 "FE" 2 "PW" 3 "PCSE" 4 "PWt" 5 "PCSEt") ///
			bycoef byopts(yrescale legend(off)) vertical yline(0) grid(none) ///
			subtitle(, size(small) color(gs0) bcolor(gs14)) ///
			p1(mcolor(gs8) ciopts(lcolor(gs8))) ///
			p2(mcolor(gs0) ciopts(lcolor(gs0))) ///
			scheme(modern) ///
			name(se_checks`i', replace)
		
		gr export "outputs/fig-se-checks-m`i'.png", replace

	}

}

/* -----------------------------------------------------------------------------
   (5) Alternative CV predictor (checking for positive effect)
   -------------------------------------------------------------------------- */

// note: using fixed effects to match the main models, but switching to random
// effects (since the predictor is largely time-invariant) returns identical
// results (easily performed by replacing 'fe' with 're' below)

eststo cv1: xtreg $cv1, fe vce(cl cty)
assert r(table)[1,7] > 0 // effect size on column 7 of Model 1 matrix

eststo cv2: xtreg $cv2, fe vce(cl cty)
assert r(table)[1,10] > 0 // effect size on column 10 of Model 2 matrix

eststo cv3: xtreg $cv3, fe vce(cl cty)
assert r(table)[1,7] > 0 // same as Model 1

eststo cv4: xtreg $cv4, fe vce(cl cty)
assert r(table)[1,10] > 0 // same as Model 2

forv i = 1/4 {
	esttab fc`i' cv`i' using "outputs/tbl-B`i'-cv-m`i'.rtf", ///
		keep(cv*) rename(cv2 cv) $xprt wide noobs
}

/* -----------------------------------------------------------------------------
   (6) Controlling for first EP election
   -------------------------------------------------------------------------- */

xtreg voter_turnout first_ep, fe vce(cl cty) // FE beta(first_ep): 6.19 (2.17)

eststo fep1: xtreg $fep1, fe vce(cl cty)
assert r(table)[4,8] > 0.1 // p-value on column 8 of Model 1 matrix

eststo fep2: xtreg $fep2, fe vce(cl cty)
assert r(table)[4,11] > 0.1 // p-value on column 12 of Model 2 matrix

eststo fep3: xtreg $fep3, fe vce(cl cty)
assert r(table)[4,8] > 0.1 // same as Model 1

eststo fep4: xtreg $fep4, fe vce(cl cty)
assert r(table)[4,11] > 0.1 // same as Model 2

forv i = 1/4 {
	esttab fep`i' using "outputs/tbl-C`i'-first-ep-m`i'.rtf", ///
		keep (first_ep) $xprt_fep wide noobs
}

/* -----------------------------------------------------------------------------
   (7-8) Sample subsets
   -------------------------------------------------------------------------- */

eststo ss11: xtreg $eq1 $ss1, fe vce(cl cty)
eststo ss12: xtreg $eq2 $ss1, fe vce(cl cty)
eststo ss13: xtreg $eq3 $ss1, fe vce(cl cty)
eststo ss14: xtreg $eq4 $ss1, fe vce(cl cty)

eststo ss21: xtreg $eq1 $ss2, fe vce(cl cty)
eststo ss22: xtreg $eq2 $ss2, fe vce(cl cty)
eststo ss23: xtreg $eq3 $ss2, fe vce(cl cty)
eststo ss24: xtreg $eq4 $ss2, fe vce(cl cty)

/* -----------------------------------------------------------------------------
   (9) Alternative DV
   -------------------------------------------------------------------------- */

corr voter_turnout vap_turnout // rho = .92

eststo va1: xtreg $va1, fe vce(cl cty)
eststo va2: xtreg $va2, fe vce(cl cty)
eststo va3: xtreg $va3, fe vce(cl cty)
eststo va4: xtreg $va4, fe vce(cl cty)

/* -----------------------------------------------------------------------------
   Plots and tables for checks 1-4 (PW only) and 7-9 (FEP, SS1, SS2, VAP)
   -------------------------------------------------------------------------- */

forv i = 1/4 {

	esttab fc`i' pw`i' ss1`i' ss2`i' va`i' ///
		using "outputs/tbl-A`i'-alt-specs-m`i'.rtf", ///
		sca(rmse) sfmt(2) ///
		indicate("Year dummies=**year") ///
		d(_cons) addn("Constant term and year dummies omitted.") ///
		mti("FE" "PW" "SS1" "SS2" "VAP") ///
		$xprt

	coefplot ///
		(fc`i', if(@ll<0 & @ul>0)) ///
		(fc`i', if(@ll>0 | @ul<0)) || ///
		(pw`i', if(@ll<0 & @ul>0)) ///
		(pw`i', if(@ll>0 | @ul<0)) || ///
		(ss1`i', if(@ll<0 & @ul>0)) ///
		(ss1`i', if(@ll>0 | @ul<0)) || ///
		(ss2`i', if(@ll<0 & @ul>0)) ///
		(ss2`i', if(@ll>0 | @ul<0)) || ///
		(va`i', if(@ll<0 & @ul>0)) ///
		(va`i', if(@ll>0 | @ul<0)) || ///
		, nooffset drop(_cons *.year) ///
		xlabel(1 "FE" 2 "PW" 3 "SS1" 4 "SS2" 5 "VAP") ///
		bycoef byopts(yrescale legend(off)) vertical yline(0) grid(none) ///
		subtitle(, size(small) color(gs0) bcolor(gs14)) ///
		p1(mcolor(gs8) ciopts(lcolor(gs8))) ///
		p2(mcolor(gs0) ciopts(lcolor(gs0))) ///
		scheme(modern) ///
		name(models`i', replace)
	
	gr export "outputs/fig-alt-specs-m`i'.png", replace

}

// done
